"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const ProstoreSsrContext_1 = require("./ProstoreSsrContext");
const storeUtils_1 = require("@proscom/prostore/lib/storeUtils");
const useConnectStore_1 = require("./useConnectStore");
const ProstoreContext_1 = require("./ProstoreContext");
/**
 * Хук, подключающий компонент к стору и запрашивающий данные
 * в соответствии с variables
 */
function useRequestStore(storeOrName, variables, options = null) {
    // Подключаем стор из контекста
    const store = ProstoreContext_1.useContextStore(storeOrName);
    // Подключение к контексту, который хранит все результаты
    const ssrContext = react_1.useContext(ProstoreSsrContext_1.ProstoreSsrContext);
    // Локальная копия состояния стора
    const [state, setState] = react_1.useState(() => {
        // Если в контексте есть значение, используем его
        return store.getValueFromContext(ssrContext);
    });
    useConnectStore_1.useConnectStore(store, setState);
    // Проверка, нужно ли запрашивать данные
    const queryCheck = storeUtils_1.checkRequestState(state, variables);
    // Мемоизированный колбек для загрузки данных
    const loadData = react_1.useCallback(() => {
        return store.loadData(variables, Object.assign({ ssr: ssrContext && ssrContext.isServer }, options)).then(() => {
            store.setContextValue(ssrContext);
        });
    }, [store, variables, options]);
    // Запрос данных, если он нужен
    react_1.useEffect(() => {
        if (queryCheck.request) {
            // Вероятность ошибки в этом месте мала, т.к.
            // основные ошибки обрабатываются в RequestStore
            // Поэтому мы ее просто залогируем
            loadData().catch((e) => console.error(e));
        }
    }, [queryCheck, loadData]);
    // Если мы сейчас в режиме серверного рендеринга, то запрашиваем данные
    if (ssrContext && ssrContext.isServer && queryCheck.request && store.ssrId) {
        ssrContext.registerPromise(loadData());
    }
    // Возвращаем то, что может быть полезно в самом компоненте
    return { check: queryCheck, state, load: loadData };
}
exports.useRequestStore = useRequestStore;
function useRequestStoreVars(storeOrName, variableCreator, args) {
    const variables = react_1.useMemo(variableCreator, args);
    return useRequestStore(storeOrName, variables);
}
exports.useRequestStoreVars = useRequestStoreVars;
//# sourceMappingURL=useRequestStore.js.map